@@ -22,6 +22,8 @@ modules = [
     "8chan",
     "8muses",
     "adultempire",
+    "agnph",
+    "ao3",
     "architizer",
     "artstation",
     "aryion",
@@ -33,6 +35,9 @@ modules = [
     "bunkr",
     "catbox",
     "chevereto",
+    "cien",
+    "civitai",
+    "cohost",
     "comicvine",
     "cyberdrop",
     "danbooru",
@@ -42,7 +47,6 @@ modules = [
     "e621",
     "erome",
     "exhentai",
-    "fallenangels",
     "fanbox",
     "fanleaks",
     "fantia",
@@ -62,6 +66,7 @@ modules = [
     "hentaifox",
     "hentaihand",
     "hentaihere",
+    "hentainexus",
     "hiperdex",
     "hitomi",
     "hotleak",
@@ -83,6 +88,7 @@ modules = [
     "keenspot",
     "kemonoparty",
     "khinsider",
+    "koharu",
     "komikcast",
     "lensdump",
     "lexica",
@@ -113,7 +119,6 @@ modules = [
     "paheal",
     "patreon",
     "philomena",
-    "photobucket",
     "photovogue",
     "picarto",
     "piczel",
