@@ -74,6 +74,21 @@ class MtimeAction(argparse.Action):
         })
 
 
+class RenameAction(argparse.Action):
+    """Configure rename post processors"""
+    def __call__(self, parser, namespace, value, option_string=None):
+        if self.const:
+            namespace.postprocessors.append({
+                "name": "rename",
+                "to"  : value,
+            })
+        else:
+            namespace.postprocessors.append({
+                "name": "rename",
+                "from": value,
+            })
+
+
 class UgoiraAction(argparse.Action):
     """Configure ugoira post processors"""
     def __call__(self, parser, namespace, value, option_string=None):
@@ -116,19 +131,24 @@ class UgoiraAction(argparse.Action):
                                       "[a] palettegen [p];[b][p] paletteuse"),
                 "repeat-last-frame": False,
             }
-        elif value in ("mkv", "copy"):
+        elif value == "mkv" or value == "copy":
             pp = {
                 "extension"        : "mkv",
                 "ffmpeg-args"      : ("-c:v", "copy"),
                 "repeat-last-frame": False,
             }
+        elif value == "zip" or value == "archive":
+            pp = {
+                "mode"             : "archive",
+            }
+            namespace.options.append(((), "ugoira", "original"))
         else:
             parser.error("Unsupported Ugoira format '{}'".format(value))
 
         pp["name"] = "ugoira"
         pp["whitelist"] = ("pixiv", "danbooru")
 
-        namespace.options.append(((), "ugoira", True))
+        namespace.options.append((("extractor",), "ugoira", True))
         namespace.postprocessors.append(pp)
 
 
@@ -173,28 +193,6 @@ def build_parser():
         action="version", version=version.__version__,
         help="Print program version and exit",
     )
-    if util.EXECUTABLE:
-        general.add_argument(
-            "-U", "--update",
-            dest="update", action="store_const", const="latest",
-            help="Update to the latest version",
-        )
-        general.add_argument(
-            "--update-to",
-            dest="update", metavar="[CHANNEL@]TAG",
-            help="Upgrade/downgrade to a specific version",
-        )
-        general.add_argument(
-            "--update-check",
-            dest="update", action="store_const", const="check",
-            help="Check if a newer version is available",
-        )
-    else:
-        general.add_argument(
-            "-U", "--update-check",
-            dest="update", action="store_const", const="check",
-            help="Check if a newer version is available",
-        )
     general.add_argument(
         "-f", "--filename",
         dest="filename", metavar="FORMAT",
@@ -216,16 +214,6 @@ def build_parser():
         dest="extractor_sources", metavar="PATH", action="append",
         help="Load external extractors from PATH",
     )
-    general.add_argument(
-        "--proxy",
-        dest="proxy", metavar="URL", action=ConfigAction,
-        help="Use the specified proxy",
-    )
-    general.add_argument(
-        "--source-address",
-        dest="source-address", metavar="IP", action=ConfigAction,
-        help="Client-side IP address to bind to",
-    )
     general.add_argument(
         "--user-agent",
         dest="user-agent", metavar="UA", action=ConfigAction,
@@ -238,6 +226,31 @@ def build_parser():
              "(ALL to delete everything)",
     )
 
+    update = parser.add_argument_group("Update Options")
+    if util.EXECUTABLE:
+        update.add_argument(
+            "-U", "--update",
+            dest="update", action="store_const", const="latest",
+            help="Update to the latest version",
+        )
+        update.add_argument(
+            "--update-to",
+            dest="update", metavar="CHANNEL[@TAG]",
+            help=("Switch to a dfferent release channel (stable or dev) "
+                  "or upgrade/downgrade to a specific version"),
+        )
+        update.add_argument(
+            "--update-check",
+            dest="update", action="store_const", const="check",
+            help="Check if a newer version is available",
+        )
+    else:
+        update.add_argument(
+            "-U", "--update-check",
+            dest="update", action="store_const", const="check",
+            help="Check if a newer version is available",
+        )
+
     input = parser.add_argument_group("Input Options")
     input.add_argument(
         "urls",
@@ -263,6 +276,11 @@ def build_parser():
         help=("Download URLs found in FILE. "
               "Delete them after they were downloaded successfully."),
     )
+    input.add_argument(
+        "--no-input",
+        dest="input", nargs=0, action=ConfigConstAction, const=False,
+        help=("Do not prompt for passwords/tokens"),
+    )
 
     output = parser.add_argument_group("Output Options")
     output.add_argument(
@@ -295,9 +313,14 @@ def build_parser():
     )
     output.add_argument(
         "-j", "--dump-json",
-        dest="jobtype", action="store_const", const=job.DataJob,
+        dest="dump_json", action="count",
         help="Print JSON information",
     )
+    output.add_argument(
+        "-J", "--resolve-json",
+        dest="dump_json", action="store_const", const=128,
+        help="Print JSON information; resolve intermediary URLs",
+    )
     output.add_argument(
         "-s", "--simulate",
         dest="jobtype", action="store_const", const=job.SimulationJob,
@@ -326,7 +349,7 @@ def build_parser():
     )
     output.add_argument(
         "--list-extractors",
-        dest="list_extractors", action="store_true",
+        dest="list_extractors", metavar="CATEGORIES", nargs="*",
         help=("Print a list of extractor classes "
               "with description, (sub)category and example URL"),
     )
@@ -347,12 +370,45 @@ def build_parser():
         help=("Write downloaded intermediary pages to files "
               "in the current directory to debug problems"),
     )
+    output.add_argument(
+        "--print-traffic",
+        dest="print_traffic", action="store_true",
+        help=("Display sent and read HTTP traffic"),
+    )
     output.add_argument(
         "--no-colors",
         dest="colors", action="store_false",
         help=("Do not emit ANSI color codes in output"),
     )
 
+    networking = parser.add_argument_group("Networking Options")
+    networking.add_argument(
+        "-R", "--retries",
+        dest="retries", metavar="N", type=int, action=ConfigAction,
+        help=("Maximum number of retries for failed HTTP requests "
+              "or -1 for infinite retries (default: 4)"),
+    )
+    networking.add_argument(
+        "--http-timeout",
+        dest="timeout", metavar="SECONDS", type=float, action=ConfigAction,
+        help="Timeout for HTTP connections (default: 30.0)",
+    )
+    networking.add_argument(
+        "--proxy",
+        dest="proxy", metavar="URL", action=ConfigAction,
+        help="Use the specified proxy",
+    )
+    networking.add_argument(
+        "--source-address",
+        dest="source-address", metavar="IP", action=ConfigAction,
+        help="Client-side IP address to bind to",
+    )
+    networking.add_argument(
+        "--no-check-certificate",
+        dest="verify", nargs=0, action=ConfigConstAction, const=False,
+        help="Disable HTTPS certificate validation",
+    )
+
     downloader = parser.add_argument_group("Downloader Options")
     downloader.add_argument(
         "-r", "--limit-rate",
@@ -360,15 +416,9 @@ def build_parser():
         help="Maximum download rate (e.g. 500k or 2.5M)",
     )
     downloader.add_argument(
-        "-R", "--retries",
-        dest="retries", metavar="N", type=int, action=ConfigAction,
-        help=("Maximum number of retries for failed HTTP requests "
-              "or -1 for infinite retries (default: 4)"),
-    )
-    downloader.add_argument(
-        "--http-timeout",
-        dest="timeout", metavar="SECONDS", type=float, action=ConfigAction,
-        help="Timeout for HTTP connections (default: 30.0)",
+        "--chunk-size",
+        dest="chunk-size", metavar="SIZE", action=ConfigAction,
+        help="Size of in-memory data chunks (default: 32k)",
     )
     downloader.add_argument(
         "--sleep",
@@ -389,21 +439,6 @@ def build_parser():
         help=("Number of seconds to wait before starting data extraction "
               "for an input URL"),
     )
-    downloader.add_argument(
-        "--filesize-min",
-        dest="filesize-min", metavar="SIZE", action=ConfigAction,
-        help="Do not download files smaller than SIZE (e.g. 500k or 2.5M)",
-    )
-    downloader.add_argument(
-        "--filesize-max",
-        dest="filesize-max", metavar="SIZE", action=ConfigAction,
-        help="Do not download files larger than SIZE (e.g. 500k or 2.5M)",
-    )
-    downloader.add_argument(
-        "--chunk-size",
-        dest="chunk-size", metavar="SIZE", action=ConfigAction,
-        help="Size of in-memory data chunks (default: 32k)",
-    )
     downloader.add_argument(
         "--no-part",
         dest="part", nargs=0, action=ConfigConstAction, const=False,
@@ -425,16 +460,6 @@ def build_parser():
         dest="download", nargs=0, action=ConfigConstAction, const=False,
         help=("Do not download any files")
     )
-    downloader.add_argument(
-        "--no-postprocessors",
-        dest="postprocess", nargs=0, action=ConfigConstAction, const=False,
-        help=("Do not run any post processors")
-    )
-    downloader.add_argument(
-        "--no-check-certificate",
-        dest="verify", nargs=0, action=ConfigConstAction, const=False,
-        help="Disable HTTPS certificate validation",
-    )
 
     configuration = parser.add_argument_group("Configuration Options")
     configuration.add_argument(
@@ -461,9 +486,19 @@ def build_parser():
     )
     configuration.add_argument(
         "--config-create",
-        dest="config_init", action="store_true",
+        dest="config", action="store_const", const="init",
         help="Create a basic configuration file",
     )
+    configuration.add_argument(
+        "--config-status",
+        dest="config", action="store_const", const="status",
+        help="Show configuration file status",
+    )
+    configuration.add_argument(
+        "--config-open",
+        dest="config", action="store_const", const="open",
+        help="Open configuration file in external application",
+    )
     configuration.add_argument(
         "--config-ignore",
         dest="config_load", action="store_false",
@@ -511,16 +546,11 @@ def build_parser():
               "domain prefixed with '/', "
               "keyring name prefixed with '+', "
               "profile prefixed with ':', and "
-              "container prefixed with '::' ('none' for no container)"),
+              "container prefixed with '::' "
+              "('none' for no container (default), 'all' for all containers)"),
     )
 
     selection = parser.add_argument_group("Selection Options")
-    selection.add_argument(
-        "--download-archive",
-        dest="archive", metavar="FILE", action=ConfigAction,
-        help=("Record all downloaded or skipped files in FILE and "
-              "skip downloading any file already in it"),
-    )
     selection.add_argument(
         "-A", "--abort",
         dest="abort", metavar="N", type=int,
@@ -533,6 +563,22 @@ def build_parser():
         help=("Stop current and parent extractor runs "
               "after N consecutive file downloads were skipped"),
     )
+    selection.add_argument(
+        "--filesize-min",
+        dest="filesize-min", metavar="SIZE", action=ConfigAction,
+        help="Do not download files smaller than SIZE (e.g. 500k or 2.5M)",
+    )
+    selection.add_argument(
+        "--filesize-max",
+        dest="filesize-max", metavar="SIZE", action=ConfigAction,
+        help="Do not download files larger than SIZE (e.g. 500k or 2.5M)",
+    )
+    selection.add_argument(
+        "--download-archive",
+        dest="archive", metavar="FILE", action=ConfigAction,
+        help=("Record all downloaded or skipped files in FILE and "
+              "skip downloading any file already in it"),
+    )
     selection.add_argument(
         "--range",
         dest="image-range", metavar="RANGE", action=ConfigAction,
@@ -573,6 +619,11 @@ def build_parser():
         dest="postprocessors", metavar="NAME", action="append", default=[],
         help="Activate the specified post processor",
     )
+    postprocessor.add_argument(
+        "--no-postprocessors",
+        dest="postprocess", nargs=0, action=ConfigConstAction, const=False,
+        help=("Do not run any post processors")
+    )
     postprocessor.add_argument(
         "-O", "--postprocessor-option",
         dest="options_pp", metavar="KEY=VALUE",
@@ -630,12 +681,24 @@ def build_parser():
         const="date|status[date]",
         help=argparse.SUPPRESS,
     )
+    postprocessor.add_argument(
+        "--rename",
+        dest="postprocessors", metavar="FORMAT", action=RenameAction, const=0,
+        help=("Rename previously downloaded files from FORMAT "
+              "to the current filename format"),
+    )
+    postprocessor.add_argument(
+        "--rename-to",
+        dest="postprocessors", metavar="FORMAT", action=RenameAction, const=1,
+        help=("Rename previously downloaded files from the current filename "
+              "format to FORMAT"),
+    )
     postprocessor.add_argument(
         "--ugoira",
-        dest="postprocessors", metavar="FORMAT", action=UgoiraAction,
-        help=("Convert Pixiv Ugoira to FORMAT using FFmpeg. "
+        dest="postprocessors", metavar="FMT", action=UgoiraAction,
+        help=("Convert Pixiv Ugoira to FMT using FFmpeg. "
               "Supported formats are 'webm', 'mp4', 'gif', "
-              "'vp8', 'vp9', 'vp9-lossless', 'copy'."),
+              "'vp8', 'vp9', 'vp9-lossless', 'copy', 'zip'."),
     )
     postprocessor.add_argument(
         "--ugoira-conv",
