@@ -9,7 +9,9 @@
 """Collection of functions that work on strings/text"""
 
 import re
+import sys
 import html
+import time
 import datetime
 import urllib.parse
 
@@ -73,7 +75,7 @@ def filename_from_url(url):
     """Extract the last part of an URL to use as a filename"""
     try:
         return url.partition("?")[0].rpartition("/")[2]
-    except (TypeError, AttributeError):
+    except Exception:
         return ""
 
 
@@ -122,7 +124,7 @@ def extract(txt, begin, end, pos=0):
         first = txt.index(begin, pos) + len(begin)
         last = txt.index(end, first)
         return txt[first:last], last+len(end)
-    except (ValueError, TypeError, AttributeError):
+    except Exception:
         return None, pos
 
 
@@ -131,7 +133,7 @@ def extr(txt, begin, end, default=""):
     try:
         first = txt.index(begin) + len(begin)
         return txt[first:txt.index(end, first)]
-    except (ValueError, TypeError, AttributeError):
+    except Exception:
         return default
 
 
@@ -141,7 +143,7 @@ def rextract(txt, begin, end, pos=-1):
         first = txt.rindex(begin, 0, pos)
         last = txt.index(end, first + lbeg)
         return txt[first + lbeg:last], first
-    except (ValueError, TypeError, AttributeError):
+    except Exception:
         return None, pos
 
 
@@ -167,7 +169,7 @@ def extract_iter(txt, begin, end, pos=0):
             last = index(end, first)
             pos = last + lend
             yield txt[first:last]
-    except (ValueError, TypeError, AttributeError):
+    except Exception:
         return
 
 
@@ -180,7 +182,7 @@ def extract_from(txt, pos=0, default=""):
             last = index(end, first)
             pos = last + len(end)
             return txt[first:last]
-        except (ValueError, TypeError, AttributeError):
+        except Exception:
             return default
     return extr
 
@@ -200,7 +202,7 @@ def parse_bytes(value, default=0, suffixes="bkmgtp"):
     """Convert a bytes-amount ("500k", "2.5M", ...) to int"""
     try:
         last = value[-1].lower()
-    except (TypeError, LookupError):
+    except Exception:
         return default
 
     if last in suffixes:
@@ -221,7 +223,7 @@ def parse_int(value, default=0):
         return default
     try:
         return int(value)
-    except (ValueError, TypeError):
+    except Exception:
         return default
 
 
@@ -231,7 +233,7 @@ def parse_float(value, default=0.0):
         return default
     try:
         return float(value)
-    except (ValueError, TypeError):
+    except Exception:
         return default
 
 
@@ -242,17 +244,28 @@ def parse_query(qs):
         for key, value in urllib.parse.parse_qsl(qs):
             if key not in result:
                 result[key] = value
-    except AttributeError:
+    except Exception:
         pass
     return result
 
 
-def parse_timestamp(ts, default=None):
-    """Create a datetime object from a unix timestamp"""
-    try:
-        return datetime.datetime.utcfromtimestamp(int(ts))
-    except (TypeError, ValueError, OverflowError):
-        return default
+if sys.hexversion < 0x30c0000:
+    # Python <= 3.11
+    def parse_timestamp(ts, default=None):
+        """Create a datetime object from a Unix timestamp"""
+        try:
+            return datetime.datetime.utcfromtimestamp(int(ts))
+        except Exception:
+            return default
+else:
+    # Python >= 3.12
+    def parse_timestamp(ts, default=None):
+        """Create a datetime object from a Unix timestamp"""
+        try:
+            Y, m, d, H, M, S, _, _, _ = time.gmtime(int(ts))
+            return datetime.datetime(Y, m, d, H, M, S)
+        except Exception:
+            return default
 
 
 def parse_datetime(date_string, format="%Y-%m-%dT%H:%M:%S%z", utcoffset=0):
